// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4635 $
#ifndef __BOARD_H__
#define __BOARD_H__

#include <vector>

#include "ChessPiece.h"
#include "Exception.h"
#include "Move.h"

extern char REV_BOARD[];

#define INITIAL_BS_LENGTH 32 // expected number of plies it'll do in a search

typedef struct
{
	int fx, fy, tx, ty; // FIXME: get x/y from p1 after undo
	ChessPiece *p1, *p2;
	MoveType mt;
	bool isPromotion;
} BoardState;

class Board final
{
protected:
	ChessPiece * b[8][8];

	std::vector<BoardState> bs;

	void move(const int fx, const int fy, const int tx, const int ty);

public:
	Board();
	Board(const Board & bIn);
	virtual ~Board();

	void clearBoard();

	void verifyBoard() const;

	void put(ChessPiece * const piece);
	void replace(ChessPiece * const piece);

	void doMove(const Move & m, BoardState *const cbs);
	void undoMove();
	const ChessPiece *findVictim(const Move & m) const;

	void setAt(const int x, const int y, ChessPiece *const p);

	inline ChessPiece * getAt(const int x, const int y) const
	{
		my_assert(y >= 0 && y < 8, "Board::getAt: y within limits");
		my_assert(x >= 0 && x < 8, "Board::getAt: x within limits");
		my_assert(b[y][x] == NULL || b[y][x] -> getX() == x, "Board::getAt: x of piece == x of board");
		my_assert(b[y][x] == NULL || b[y][x] -> getY() == y, "Board::getAt: y of piece == y of board");

		return b[y][x];
	}
};

bool isEpMove(const Board *const b, const Move & m);

#endif
