// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4872 $
#ifndef __BOOK_H__
#define __BOOK_H__

#include <atomic>
#include <condition_variable>
#include <map>
#include <random>
#include <thread>

#include "config.h"
#include "Gen.h"
#include "IO.h"
#include "Scene.h"
#include "Utils.h"

typedef enum { BOOK_EVAL, BOOK_MOVE } book_type_t;

typedef enum { BR_BUSY, BR_OK, BR_OK_TT, BR_ERR } book_result_t;

typedef enum { WIN_WHITE, WIN_BLACK, WIN_DRAW, WIN_UNKNOWN /* low-speed tt */ } winner_t;

typedef struct
{
	std::string fen;
	PlayerColor c;
} bquestion_t;

typedef struct
{
	 VectorType<PVnode_t> *pv;
	 bool is_final_answer;
} book_answer_t;

class Book
{
protected:
	IO *const io;
	std::thread *th;
	std::atomic_bool terminate;
	//
	std::atomic_int id;
	//
	rw_lock *rm_lock;
	std::condition_variable workAdded;
	std::map<int, bquestion_t> in_queue;
	std::map<int, book_answer_t> results;

	std::random_device rd;
	MERS_RNG *mt;

	void putResult(const int cur_id, const Move & m, const int eval, const bool isFinalAnswer, const bool valid);
	void putResult(const int cur_id, VectorType<PVnode_t> *const ml, const bool isFinalAnswer, const bool valid);

public:
	Book(IO *const io);
	virtual ~Book();

	int getRandom(int max_excl);

	virtual int ask(Scene & s, const PlayerColor c);
	virtual void stopAllAsks();

	virtual book_result_t getResult(const int idIn, const Scene *const s, const PlayerColor c, VectorType<PVnode_t> **const ml, bool *const is_final_answer);

	virtual void learn(const Board & b, const PlayerColor c, const Move & epMove, const Move & bestMove, const int depth, const winner_t win);

	virtual void operator()() = 0;
};

Move moveStringToMove(const std::string & str, const PlayerColor pc);

#endif
