// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4871 $
// WDL book
#include "Tweaks.h"
#include <limits.h>
#include "config.h"
#ifdef WITH_MYSQL_BOOKS
#include "DBBookEval.h"
#include "Utils.h"
#include "Move.h"
#include "MoveType.h"

DBBookEval::DBBookEval(IO *const ioIn, const std::string & host, const std::string & db, const std::string & user, const std::string & password, const std::string & db_tableIn) : DBBook(ioIn, host, db, user, password, db_tableIn)
{
	log(ll_ok, "DBBookEval started");

	th = new std::thread(std::ref(*this));
}

DBBookEval::~DBBookEval()
{
}

void DBBookEval::operator()()
{
	log(ll_ok, "DBBookEval::operator() starting");

	std::mutex mtx;
	std::unique_lock<std::mutex> lk(mtx);

	bool first = true;

	for(;!terminate;)
	{
		bool wasFirst = first;

		if (first)
			first = false;
		else
		{
			workAdded.wait(lk);

			if (terminate)
				break;
		}

		int cur_id = -1;
		std::string workFen;

		w_lock(rm_lock);
		auto iq_it = in_queue.begin();

		if (iq_it != in_queue.end())
		{
			cur_id = iq_it -> first;
			workFen = iq_it -> second.fen;
		}

		w_ulock(rm_lock);

		if (cur_id == -1)
		{
			if (!wasFirst)
				log(ll_err, "DBBookEval: nothing in queue?");
			continue;
		}

		log(ll_ok, "DBBookEval: searching for %s", workFen.c_str());

		std::string query = myformat("SELECT pv, IFNULL(eval, 0) AS eval FROM %s WHERE fen='%s' LIMIT 1", db_table.c_str(), workFen.c_str());

		std::vector<std::vector<std::string> > query_results;
		if (!do_query(handle, query, &query_results))
		{
			log(ll_err, "DBBookEval: database access failed!!");
			putResult(cur_id, invalidMove, INT_MIN, false, false);
			continue;
		}

		long unsigned int nResults = query_results.size();
		log(ll_ok, "DBBookEval: %d results from database", nResults);

		std::vector<std::string> *moves = NULL;
		int eval = INT_MIN;
		if (nResults == 1) {
			moves = split(query_results.at(0).at(0), " ");
			eval = atoi(query_results.at(0).at(1).c_str());
		}

		if (moves && !moves -> empty())
		{
			VectorType<PVnode_t> *ml = new VectorType<PVnode_t>();

			for(size_t i=0; i<moves -> size(); i++)
			{
				const std::string move = moves -> at(i);

				if (i == 0)
					log(ll_ok, "DBBookEval: Move returned: %s (%d)", move.c_str(), eval);

				std::vector<std::string> *fen_parts = split(workFen, " ");
				Move m = moveStringToMove(move, fen_parts -> at(1) == "b" ? BLACK : WHITE);
				delete fen_parts;

				push_back(ml, 0, m, eval, MS_BOOK );
			}

			log(ll_ok, "DBBookEval: result stored");

			putResult(cur_id, ml, true, true);
		}
		else
		{
			putResult(cur_id, invalidMove, INT_MIN, false, false);

			log(ll_ok, "DBBookEval: no result found");
		}

		delete moves;
	}

	log(ll_ok, "DBBookEval::operator(): terminating");
}
#endif
