// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4871 $
// WDL book
#include "Tweaks.h"
#include <limits.h>
#include "config.h"
#ifdef WITH_MYSQL_BOOKS
#include "DBBookWDL.h"
#include "Utils.h"
#include "Move.h"
#include "MoveType.h"

DBBookWDL::DBBookWDL(IO *const ioIn, const std::string & host, const std::string & db, const std::string & user, const std::string & password, const std::string & db_tableIn) : DBBook(ioIn, host, db, user, password, db_tableIn)
{
	log(ll_ok, "DBBookWDL started");

	th = new std::thread(std::ref(*this));
}

DBBookWDL::~DBBookWDL()
{
}

void DBBookWDL::operator()()
{
	log(ll_ok, "DBBookWDL::operator() starting");

	std::mutex mtx;
	std::unique_lock<std::mutex> lk(mtx);

	bool first = true;

	for(;!terminate;)
	{
		bool wasFirst = first;

		if (first)
			first = false;
		else
		{
			workAdded.wait(lk);

			if (terminate)
				break;
		}

		int cur_id = -1;
		std::string workFen;

		w_lock(rm_lock);

		auto iq_it = in_queue.begin();

		if (iq_it != in_queue.end())
		{
			cur_id = iq_it -> first;
			workFen = iq_it -> second.fen;
		}

		w_ulock(rm_lock);

		if (cur_id == -1)
		{
			if (!wasFirst)
				log(ll_err, "DBBookWDL: nothing in queue?");
			continue;
		}

		log(ll_ok, "DBBookWDL: searching for %s", workFen.c_str());

		std::string query = myformat("SELECT move, ((win + 1.9208) / (win + loose) - 1.96 * SQRT((win * loose) / (win + loose) + 0.9604) / (win + loose)) / (1 + 3.8416 / (win + loose)) AS ci_lower_bound FROM %s WHERE fen_before='%s' AND win + loose >= %d AND win > loose ORDER BY ci_lower_bound DESC LIMIT 1", db_table.c_str(), workFen.c_str(), BOOK_WDL_MIN_HITS);

		std::vector<std::vector<std::string> > query_results;
		if (!do_query(handle, query, &query_results))
		{
			log(ll_err, "DBBookWDL: database access failed!!");
			putResult(cur_id, invalidMove, INT_MIN, false, false);
			continue;
		}

		long unsigned int nResults = query_results.size();
		log(ll_ok, "DBBookWDL: %d results from database", nResults);

		if (nResults == 1)
		{
			std::string move = query_results.at(0).at(0);
			log(ll_ok, "DBBookWDL: Move returned: %s", move.c_str());

			std::vector<std::string> *fen_parts = split(workFen, " ");
			Move m = moveStringToMove(move, fen_parts -> at(1) == "b" ? BLACK : WHITE);
			delete fen_parts;

			putResult(cur_id, m, INT_MIN, true, true);

			log(ll_ok, "DBBookWDL: result stored");
		}
		else
		{
			putResult(cur_id, invalidMove, INT_MIN, false, false);

			log(ll_ok, "DBBookWDL: no result found");
		}
	}

	log(ll_ok, "DBBookWDL::operator() terminating");
}
#endif
