#include <stdint.h>
#include <stdio.h>
#include <string>
#include <string.h>

#include "Utils.h"
#include "DetectBookFile.h"

book_fileformat_t detectBookType(const std::string & filename)
{
	FILE *fh = fopen(filename.c_str(), "rb");
	if (!fh)
		return bt_other;

	uint8_t buffer[4096];
	if (fread(buffer, 1, sizeof buffer, fh) != sizeof buffer)
		return bt_other;

	// sqlite3
	char sqlite3Header[] = "SQLite format 3";
	if (memcmp(buffer, sqlite3Header, 16) == 0)
		return bt_sqlite3;

	// polyglot
	fseek(fh, 0, SEEK_END);
	size_t len = ftell(fh);

	if (len % 16)
		return bt_other;

	// check if hashes are in increasing order
	uint64_t prev = 0;
	for(size_t i=0; i<(sizeof buffer/16); i++) {
		uint64_t cur = int_from_mem(&buffer[i * 16], 8);

		if (cur < prev)
			return bt_other;
	}

	fclose(fh);

	return bt_polyglot;
}
