// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4539 $
#ifndef __GEN_H__
#define __GEN_H__

#include <limits.h>
#if (__WORDSIZE == 64)
#define IS_64B
#endif

#ifdef IS_64B
#define MERS_RNG std::mt19937_64
#else
#define MERS_RNG std::mt19937
#endif

#define INFINITE 32000
#define MS_INF 2000000000

#define ULI unsigned long int

#include <algorithm>
#include <stdint.h>

uint64_t get_ts_ms();
typedef struct _calc_stats_
{
	uint64_t started_at;
	uint64_t cnt, tpt_hit_cnt, qcnt, qtpt_hit_cnt, qqtpt_hit_cnt, q_limit, tt_err, tt_d0, tt_hit_type[4], tt_depth_ok;
	uint64_t cnt_cutoff_index, cnt_cutoff, max_depth_reached;
	uint64_t null_move, null_move_hit;
	uint64_t se_query[2], se_hit[2][4];
	uint64_t threads_created, iid, iid_hit;
	uint64_t lmr_ok, lmr_fail, qstand_pat_bco;
	uint64_t pif_size, pif_size_n;
	uint64_t r_extensions, draw, mate, q_nm, blind, blind_total, history_hits;
	uint64_t ptt_hit, ptt_mis;
	uint64_t evals;
	int max_depth;

	void addCounts(struct _calc_stats_ *const cs) const
	{
		cs -> cnt += cnt;
		cs -> qcnt += qcnt;
		cs -> tpt_hit_cnt += tpt_hit_cnt;
		cs -> qtpt_hit_cnt += qtpt_hit_cnt;
		cs -> qqtpt_hit_cnt += qqtpt_hit_cnt;
		cs -> q_limit += q_limit;
		cs -> max_depth_reached += max_depth_reached;
		cs -> null_move += null_move;
		cs -> null_move_hit += null_move_hit;
		cs -> threads_created += threads_created;
		cs -> iid += iid;
		cs -> iid_hit += iid_hit;
		for(int i=0; i<2; i++) {
			cs -> se_query[i] += se_query[i];

			for(int k=0; k<4; k++)
				cs -> se_hit[i][k] += se_hit[i][k];
		}
		cs -> lmr_ok += lmr_ok;
		cs -> lmr_fail += lmr_fail;
		cs -> max_depth = std::max(cs -> max_depth, max_depth);
		cs -> tt_err += tt_err;
		cs -> tt_d0 += tt_d0;
		cs -> cnt_cutoff_index += cnt_cutoff_index;
		cs -> cnt_cutoff += cnt_cutoff;
		cs -> pif_size += pif_size;
		cs -> pif_size_n += pif_size_n;
		cs -> r_extensions += r_extensions;
		cs -> qstand_pat_bco += qstand_pat_bco;
		cs -> draw += draw;
		cs -> mate += mate;
		cs -> q_nm += q_nm;
		cs -> blind += blind;
		cs -> blind_total += blind_total;
		cs -> history_hits += history_hits;
		cs -> ptt_hit += ptt_hit;
		cs -> ptt_mis += ptt_mis;
		for(int i=0; i<4; i++)
			cs -> tt_hit_type[i] += tt_hit_type[i];
		cs -> tt_depth_ok += tt_depth_ok;
		cs -> evals += evals;
	}

	void setStartedAt() { started_at = get_ts_ms(); }
	uint64_t getStartedAt() const { return started_at; }
} calc_stats_t;

#endif
