// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4836 $
#include "Tweaks.h"
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "Exception.h"
#include "Utils.h"
#include "IO_AutoPlay.h"
#include "Brain.h"
#include "History.h"

IO_AutoPlay::IO_AutoPlay(const std::map<std::string, std::string> & parametersIn, Tpt *const ttIn, Ptt *const pttIn) : IO(parametersIn, ttIn, pttIn)
{
//	fen = cfgGetString(parametersIn, "fen", "");
}

IO_AutoPlay::~IO_AutoPlay()
{
}

void IO_AutoPlay::loop()
{
	PlayerColor c = WHITE;

	Scene *s = NULL;
	new_Scene(&s, &c);

	Move epMove = invalidMove, lastOM = invalidMove;

	std::vector<uint64_t> history;

	int maxNMoves = cfgGetInt(parameters, "max_n_moves", 0), moveNr = 1;
	if (maxNMoves)
		log(ll_ok, "Maximum number of moves: %d", maxNMoves);

	int cfgDepth = cfgGetInt(parameters, "max_depth", -1);
	int cfgThink = cfgGetInt(parameters, "max_time", -1);

	int nMoveRandom = cfgGetInt(parameters, "n_move_random", 0);

	for(; maxNMoves == 0 || moveNr < maxNMoves;)
	{
		if (verbose)
			printf("Move: %d/%d\n", moveNr, c);

		log(ll_ok, "Move: %d", moveNr);
		if (stopFlag)
		{
			log(ll_ok, "Abort");
			break;
		}

		s -> generateMovelists();
		s -> partialValidateMoves(c);

		if (s -> isCheckMate(WHITE) || s -> isCheckMate(BLACK))
		{
			log(ll_ok, "Checkmate");
			break;
		}

		if (s -> isStaleMate(WHITE) || s -> isStaleMate(BLACK))
		{
			log(ll_ok, "Stale mate");
			break;
		}

		if (s -> isInsufficientMaterialDraw())
		{
			log(ll_ok, "Insufficient material");
			break;
		}

		int value = -32768;

		if (stopFlag.exchange(false))
			break;

		zobrist_t hash = b -> calcZobrist(s -> getBoardConst(), c, epMove);
		if (addToHistory(&history, hash.newHash))
		{
			log(ll_ok, "3-rule");
			break;
		}

		if (moveNr <= nMoveRandom) {
			VectorType<Move> *const moves = s -> getMoveList(c);

			size_t nr = 0;
			int i = 0;
			for(; i<10; i++) {
				nr = rand() % moves -> size();

				if (s -> validateMoves(c, nr))
                          		break;
			}

			if (i == 10)
				goto calc;

			BoardState bs;
			s -> doMove(moves -> at(nr), &bs);
		}
		else {
		calc:
			ei.flag = false;
			VectorType<PVnode_t> *ml = NULL;
			uint64_t cnts = 0;
			b -> calculateMove(moveNr, *s, c, cfgDepth, cfgThink, cfgThink, NULL, &history, epMove, &ml, &value, &ei, &cnts, invalidMove, lastOM);

			if (ml -> empty())
			{
				log(ll_err, "bestmove a1a1");
				break;
			}
			else
			{
				log(ll_ok, "bestmove %s", moveToUciMove(ml -> at(0).m).c_str());

				if (verbose)
					emitPvDetails(s, c, ml -> at(0).m);

				if (ml -> size() >= 2)
					log(ll_ok, " ponder %s", moveToUciMove(ml -> at(1).m).c_str());

				if (isEpMove(s -> getBoardConst(), ml -> at(0).m))
					epMove = ml -> at(0).m;
				else
					epMove = invalidMove;

				lastOM = ml -> at(0).m;

				BoardState bs;
				s -> doMove(ml -> at(0).m, &bs);

				c = opponentColor(c);
			}

			delete ml;
		}

		if (c == WHITE)
			moveNr++;
	}

	log(ll_ok, "finished");

	delete s;
}
