// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4786 $
#include "Tweaks.h"
#include "config.h"
#ifdef W_READLINE
#include <ctype.h>
#include <stdio.h>
#include <readline/readline.h>

#include "Brain.h"
#include "Main.h"
#include "History.h"
#include "IO_Console.h"

IO_Console::IO_Console(const std::map<std::string, std::string> & parametersIn, Tpt *const ttIn, Ptt *const pttIn) : IO(parametersIn, ttIn, pttIn)
{
	maxIt = cfgGetInt(parameters, "max_depth", 0);
	duration = cfgGetInt(parameters, "max_time", 5000);
}

IO_Console::~IO_Console()
{
}

void IO_Console::loop()
{
	emitCmd("%s\n", versionStr);

	PlayerColor c = WHITE;
	Scene *s = NULL;
	new_Scene(&s, &c);

	int moveNr = 1;

	Move epMove = invalidMove, lastOM = invalidMove;
	std::vector<uint64_t> history;

	for(;;)
	{
		s -> generateMovelists();
		s -> partialValidateMoves(c);

		if (s -> isCheckMate(WHITE) || s -> isCheckMate(BLACK))
		{
			emitCmd("Checkmate\n");
			break;
		}

		if (s -> isStaleMate(WHITE) || s -> isStaleMate(BLACK))
		{
			emitCmd("Stale mate\n");
			break;
		}

		if (s -> isInsufficientMaterialDraw())
		{
			emitCmd("Insufficient material\n");
			break;
		}

		zobrist_t hash = b -> calcZobrist(s -> getBoardConst(), c, epMove);

		VectorType<PVnode_t> *ml = NULL;

		dumpBoard(hash, s, DB_P, c);
		if (verbose)
			emitCmd("FEN: %s", genFen(s, false, c).c_str());
		emitCmd("Move nr.: %d, color: %s\n\n", moveNr, c == WHITE ? "white" : "black");

		if (addToHistory(&history, hash.newHash))
		{
			emitCmd("3-rule\n");
			break;
		}

		if (c == WHITE)
		{
			Move m = invalidMove;

			for(;;)
			{
				for(;;)
				{
					char *moveStr = readline("Your move: ");
					if (!moveStr)
						return;

					std::string clean = trim(std::string(moveStr));
					free(moveStr);

					VectorType<Move> *mlValid = s -> getMoveList(c);

					if (SANToMove(clean, *s, c, mlValid, &m))
						break;

					emitCmd("Did not understand \"%s\": it may be an invalid move.\n", clean.c_str());

					std::string moves;
					for(size_t i=0; i<mlValid -> size(); i++)
						moves += moveToString(mlValid -> at(i)) + " ";

					emitCmd("Valid moves: %s\n", moves.c_str());
				}

				emitCmd("Interpreted as: %s\n", moveToString(m).c_str());

			wrong_char1:
				emitCmd("Ok? (press y or n + enter)\n");

			wrong_char2:
				int chr = getchar();
				if (chr == 'y' || chr == 'Y')
					break;
				if (chr == 'n' || chr == 'N')
					continue;
				if (chr == 13 || chr == 10)
					goto wrong_char2;
				goto wrong_char1;
			}

			ml = new VectorType<PVnode_t>();
			push_back(ml, 0, m, 0, MS_DUMMY);
		}
		else
		{
			printf("Please wait... ");

			if (duration > 0)
				printf("(%.1f (or less) seconds)", duration / 1000.0);
			else if (maxIt > 0)
				printf("(search depth %d)", maxIt);

			printf("\n");

			ei.flag = false;

			int value = -32768;
			uint64_t cnts = 0;
			b -> calculateMove(moveNr, *s, c, maxIt, duration, duration, NULL, &history, epMove, &ml, &value, &ei, &cnts, invalidMove, lastOM);

			std::string smiley = ":-|";

			if (value < -10)
				smiley = ":-(";
			else if (value > 10)
				smiley = ":-)";
			else if (value < 0)
				smiley = ":-\\";
			else if (value > 0)
				smiley = ":-/";
			else
				smiley = ":-|";

			if (ml -> empty())
			{
				emitCmd("Can't think of any move to do\n");
				break;
			}
			else
			{
				emitCmd("My move: %s (%d) %s\n", moveToString(ml -> at(0).m).c_str(), value, smiley.c_str());
			}
		}

		Move curm = ml -> at(0).m;
		if (s -> getBoardConst() -> getAt(curm.u.m.fx, curm.u.m.fy) -> getType() == PAWN && abs(curm.u.m.ty - curm.u.m.fy) == 2)
			epMove = curm;
		else
			epMove = invalidMove;

		BoardState bs;
		s -> doMove(curm, &bs);

		lastOM = curm;

		delete ml;

		if (c == WHITE)
			c = BLACK;
		else
		{
			c = WHITE;
			moveNr++;
		}
	}
}

void IO_Console::emitDebug(const std::string what, ...) const
{
	char *msg = NULL;

       	va_list ap;
       	va_start(ap, what);
	(void)vasprintf(&msg, what.c_str(), ap);
	va_end(ap);

	log(ll_ok, "DEBUG: " + std::string(msg));

	free(msg);
}

void IO_Console::emitCmd(const std::string what, ...) const
{
	char *msg = NULL;

       	va_list ap;
       	va_start(ap, what);
	(void)vasprintf(&msg, what.c_str(), ap);
	va_end(ap);

	log(ll_ok, "DEBUG: " + std::string(msg));
	printf("%s\n", msg);

	free(msg);
}
#endif
