// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4786 $
#include "Tweaks.h"
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "Exception.h"
#include "Utils.h"
#include "IO.h"
#include "IO_EPD.h"
#include "Brain.h"

IO_EPD::IO_EPD(const std::map<std::string, std::string> & parametersIn, Tpt *const ttIn, Ptt *const pttIn) : IO(parametersIn, ttIn, pttIn)
{
	maxIt = cfgGetInt(parameters, "max_depth", -1);
	duration = cfgGetInt(parameters, "max_time", 1000);
}

IO_EPD::~IO_EPD()
{
}

void IO_EPD::emitCmd(const std::string & what) const
{
	fprintf(stderr, "%s\n", what.c_str());
}

void IO_EPD::emitDebug(const std::string what, ...) const
{
	char *msg = NULL;

       	va_list ap;
       	va_start(ap, what);
	(void)vasprintf(&msg, what.c_str(), ap);
	va_end(ap);

	fprintf(stderr, "%s\n", msg);

	free(msg);
}

void IO_EPD::loop()
{
	int n_skipped = 0, n_found = 0, n_total = 0;
	uint64_t start_ts = get_ts_ms();

	uint64_t t_cnts = 0;
	for(;!stopFlag;)
	{
		char buffer[4096];

		if (!fgets(buffer, sizeof buffer, stdin))
			break;

		char *lf = strchr(buffer, '\n');
		if (lf)
			*lf = 0x00;

		char *cr = strchr(buffer, '\r');
		if (cr)
			*cr = 0x00;

		log(ll_ok, buffer);

		if (buffer[0] == 0x00 || buffer[0] == '#' || buffer[0] == ';' || (buffer[0] == '/' && buffer[1] == '/'))
			continue;

		int cur_duration = duration;
		if (memcmp(&buffer[0], "think_time=", 11) == 0) {
			cur_duration = atoi(&buffer[11]);

			char *sp = strchr(buffer, ' ');
			if (!sp)
				continue;

			memmove(&buffer[0], sp + 1, strlen(buffer) - (sp - buffer));
		}

		std::string parse;

		size_t n = strlen(buffer);
		for(size_t i=0; i<n; i++)
		{
			if (buffer[i] == ';')
				parse += " ; ";
			else
				parse += buffer[i];
		}

		std::vector<std::string> *parts = split(parse, " ");

		n_total++;

		if (parts -> size() >= 5 && parts -> at(4) == "bm")
		{
			PlayerColor color = parts -> at(1) == "w" ? WHITE : BLACK;

			// FIXME 4/5?
			const std::string fen = parts -> at(0) + " " + parts -> at(1) + " " + parts -> at(2) + " " + parts -> at(3);
			Scene s(fen);
			s.generateMovelists();

			VectorType<Move> endMoves;

			bool fail = false;
			size_t i = 5;
			while(i < parts -> size() && parts -> at(i) != ";")
			{
				Move m;

				fprintf(stderr, "%s ", parts -> at(i).c_str());

				if (!SANToMove(parts -> at(i), s, color, s.getMoveList(color), &m))
				{
					fprintf(stderr, "Failed parsing \"%s\"\n", parts -> at(i).c_str());
					fail = true;
					break;
				}

				endMoves.push_back(m);

				i++;
			}

			if (fail)
				continue;

			uint64_t start = get_ts_ms();

			ei.flag = false;

			std::vector<uint64_t> history;
			int value = 0;
			VectorType<PVnode_t> *ml = NULL;
			// FIXME epmove
			uint64_t cur_cnts = 0;
			b -> calculateMove(0, s, color, maxIt, cur_duration, cur_duration, &endMoves, &history, invalidMove, &ml, &value, &ei, &cur_cnts, invalidMove, invalidMove);
			t_cnts += cur_cnts;

			uint64_t finish = get_ts_ms();

			if (ml -> empty() || indexOf(endMoves, ml -> at(0).m) == -1)
				fprintf(stderr, "not found\n");
			else {
				fprintf(stderr, "%s: found %s (%fs)\n", buffer, moveToString(ml -> at(0).m).c_str(), double(finish - start) / 1000.0);
				n_found++;
			}

			delete ml;
		}
		else
		{
			fprintf(stderr, "Skipping \"%s\"\n", buffer);
			n_skipped++;
		}

		fflush(NULL);

		delete parts;
	}

	uint64_t end_ts = get_ts_ms();
	double took = (end_ts - start_ts) / 1000.0;

	printf("{ \"skipped\" : %d, \"found\" : %d, \"total\" : %d, \"took\" : %f, \"version\" : \"" VERSION " " NAME_EXTRA "\", \"avg-nps\" : %f }\n", n_skipped, n_found, n_total, took, t_cnts / took);
}
