// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4829 $
#include "Tweaks.h"
#include <ctype.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "Exception.h"
#include "Utils.h"
#include "IO.h"
#include "IO_perft.h"
#include "Brain.h"

IO_perft::IO_perft(const std::map<std::string, std::string> & parametersIn, Tpt *const ttIn, Ptt *const pttIn) : IO(parametersIn, ttIn, pttIn)
{
	maxCount = cfgGetInt(parametersIn, "max_count", -1);
	search = cfgGetInt(parametersIn, "search", -1);
}

IO_perft::~IO_perft()
{
}

void IO_perft::emitCmd(const std::string & what) const
{
	fprintf(stderr, "%s\n", what.c_str());
}

void IO_perft::emitDebug(const std::string what, ...) const
{
	char *msg = NULL;

       	va_list ap;
       	va_start(ap, what);
	(void)vasprintf(&msg, what.c_str(), ap);
	va_end(ap);

	fprintf(stderr, "%s\n", msg);

	free(msg);
}

void IO_perft::loop()
{
	int n_rows = 0, n_ok = 0, n_fail = 0;
	uint64_t start_ts = get_ts_ms();

	for(;!stopFlag && !ei.flag;)
	{
		char buffer[4096];

		if (!fgets(buffer, sizeof buffer, stdin))
			break;

		char *lf = strchr(buffer, '\n');
		if (lf)
			*lf = 0x00;

		char *cr = strchr(buffer, '\r');
		if (cr)
			*cr = 0x00;

		log(ll_ok, buffer);

		if (buffer[0] == 0x00 || buffer[0] == '#' || buffer[0] == ';' || (buffer[0] == '/' && buffer[1] == '/'))
			continue;

		std::string parse;

		size_t n = strlen(buffer);
		for(size_t i=0; i<n; i++)
		{
			if (buffer[i] == ';')
				parse += " ; ";
			else
				parse += buffer[i];
		}

		std::vector<std::string> *parts = split(parse, ";");
		std::string fen = trim(parts -> at(0));

		n_rows++;

		std::vector<std::string> *fparts = split(fen, " ");
		PlayerColor color = fparts -> at(1) == "w" ? WHITE : BLACK;
		delete fparts;

		if (search >= 0) {
			Scene s(fen.c_str());
			s.generateMovelists();

			uint64_t cnt = 0;

			ei.flag = false;
			int score = 0;
			VectorType<PVnode_t> *pv = NULL;
			b -> calculateMove(0, s, color, search, -1, -1, NULL, NULL, invalidMove, &pv, &score, &ei, &cnt, invalidMove, invalidMove);

			printf("%s\t%lu\n", fen.c_str(), (ULI)cnt);
		}
		else {
			printf("%s:\n", fen.c_str());

			for(size_t i=1; i<parts -> size(); i++) {
				if (stopFlag || ei.flag)
					 break;

				Scene s(fen.c_str());

				std::vector<std::string> *dparts = split(trim(parts -> at(i)).substr(1), " ");
				int depth = atoi(dparts -> at(0).c_str());

				perft_counts_t pc;
				memset(&pc, 0x00, sizeof pc);

				long int okCnt = atoll(dparts -> at(1).c_str());
				if (okCnt <= maxCount || maxCount == -1) {
					uint64_t cur_start_ts = get_ts_ms();

					s.generateMovelists();
					long int cnt = b -> perft(s, depth, color, &pc, false);

					uint64_t cur_end_ts = get_ts_ms();
					double cur_took = (cur_end_ts - cur_start_ts) / 1000.0;

#if 1
					int eval = -1; // evaluate(&s, color, 0); // FIXME

					printf("\t%d: %lu %s (%fs) (eval %d)\n", depth, cnt, okCnt == cnt ? "OK" : myformat("fail (%lu)", okCnt).c_str(), cur_took, eval);
#endif

					if (okCnt == cnt)
						n_ok++;
					else
						n_fail++;
				}

				delete dparts;
			}

			printf("\n");
		}

		fflush(NULL);

		delete parts;
	}

	uint64_t end_ts = get_ts_ms();
	double took = (end_ts - start_ts) / 1000.0;

	printf("took: %fs, n_rows: %d, n_tests: %d (%d ok), ok: %.2f%%\n", took, n_rows, n_ok + n_fail, n_ok, double(n_ok * 100) / (n_ok + n_fail));
}
