// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4641 $
#include "Tweaks.h"
#include "config.h"
#include "Move.h"
#include "Utils.h"
#include "IO.h"
#include "Exception.h"

bool moveCompareTo(const Move & a, const Move & b)
{
	my_assert(sizeof(Move::_u_::m) == 4, "Move structure must be exactly 4 bytes with no padding");

	return a.u.mui == b.u.mui;
}

void dumpMoveList(const VectorType<Move> *const moveList)
{
	for(size_t idx=0; idx<moveList -> size(); idx++)
		printf("%2d] %s\n", (int)(idx + 1), moveToString(moveList -> at(idx)).c_str());
}

std::string moveToString(const Move & m, const bool dash, const Board *const b, const bool isCheck)
{
	std::string out;

	if (b) {
		ChessPieceType cpt = b -> getAt(m.u.m.fx, m.u.m.fy) -> getType();

		if (cpt != PAWN)
			out += toupper(typeToChar(cpt));
	}

	if (m.u.m.mt == CASTLING) {
		if (m.u.m.tx == 2)
			out = "o-o-o";
		else if (m.u.m.tx == 6)
			out = "o-o";
		else
			out += "!?";
	}
	else if (m.u.m.mt == EN_PASSANT || m.u.m.mt == REGULAR) {
		const char *d = m.u.m.isCatchMove ? "x" : "-";

		char tt[2] = { 0x00, 0x00 };
		if (dash && b) {
			if (m.u.m.isCatchMove) {
				const ChessPiece *const cv = b -> findVictim(m);
				tt[0] = toupper(typeToChar(cv -> getType()));
			}
		}

		out += myformat("%c%c%s%s%c%c", m.u.m.fx + 'a', m.u.m.fy + '1', dash ? d : "", tt, m.u.m.tx + 'a', m.u.m.ty + '1');
	}
	else {
		out += "?!";
	}

	if (m.u.m.promoteTo != NONE) {
		out += "=";

		if (m.u.m.promoteTo == QUEEN)
			out += "q";
		else if (m.u.m.promoteTo == BISHOP)
			out += "b";
		else if (m.u.m.promoteTo == KNIGHT)
			out += "n";
		else if (m.u.m.promoteTo == ROOK)
			out += "r";
		else if (m.u.m.promoteTo == KING)
			out += "k";
		else if (m.u.m.promoteTo == PAWN)
			out += "p";
		else
			out += "?!!";
	}

	if (isCheck)
		out += "+";

	return out;
}

Move * moveOrNull(Move *const m)
{
	if (isValidMove(*m))
		return m;

	return NULL;
}

bool isValidMove(const VectorType<Move> *const restrict ml, const Move & m)
{
	if (!isValidMove(m))
		return false;

	return indexOf(*ml, m) != -1;
}

bool isMove(const Move & m, const char *const str)
{
	return m.u.m.fx == tolower(str[0]) - 'a' &&
		m.u.m.fy == tolower(str[1]) - '1' &&
		m.u.m.tx == tolower(str[2]) - 'a' &&
		m.u.m.ty == tolower(str[3]) - '1';
}

bool isValidMove(const Move & m)
{
	my_assert(m.u.m.fx < 8 || m.u.m.fx == INVALID_MOVE, "move sanity");
	my_assert(m.u.m.fy < 8, "move sanity");
	my_assert(m.u.m.tx < 8, "move sanity");
	my_assert(m.u.m.ty < 8, "move sanity");
	my_assert(m.u.m.mt == REGULAR || m.u.m.mt == CASTLING || m.u.m.mt == EN_PASSANT, "move sanity");
	my_assert(m.u.m.promoteTo < 6 || m.u.m.promoteTo == NONE, "move sanity");
	my_assert(m.u.m.mt != CASTLING || m.u.m.isCatchMove == false, "move sanity");

	return m.u.m.fx != INVALID_MOVE;
}
