// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 5116 $
#include "Tweaks.h"
#include <stdint.h>

#include "config.h"
#include "Exception.h"
#include "Board.h"
#include "ChessPiece.h"
#include "Move.h"
#include "MoveSort.h"
#include "Tweaks.h"
#include "ALG_position.h"
#include "Gen.h"

char REV_MOVESORT[] = "MoveSort $Revision: 5116 $";

int calcSortValue(const Move *const restrict m, const Board *const restrict b, const search_data_t *const sd, const int phase)
{
        const ChessPiece *attacker = b -> getAt(m -> u.m.fx, m -> u.m.fy); // from

        int promoteValue = ChessPiece::evalVal[m -> u.m.promoteTo];

        int score = promoteValue << 18;

        if (m -> u.m.isCatchMove) {
		const ChessPiece *victim = b -> findVictim(*m);
		int victimValue = victim -> getEvalVal() - sd -> s -> getIsProtected(victim) * 50;
		score += victimValue << 18;

		int attackerValue = attacker -> getEvalVal();

                score += (ChessPiece::evalVal[QUEEN] - attackerValue) << 8;
	}
        else {
                const ChessPieceType cpt = attacker -> getType();
                const PlayerColor c = attacker -> getColor();

                score += evalPositions(m -> u.m.tx, m -> u.m.ty, cpt, c, phase) -
                         evalPositions(m -> u.m.fx, m -> u.m.fy, cpt, c, phase);

#ifdef WITH_HBT
                score += sd -> hbt[c][m -> u.m.fy * 8 + m -> u.m.fx][m -> u.m.ty * 8 + m -> u.m.tx] << 8;
#endif
        }

        return score;
}

int moveSortVictimType(const void *restrict pm1, const void *restrict pm2)
{
	const Move *m1 = (const Move *)pm1;
	const Move *m2 = (const Move *)pm2;

	return m2 -> score - m1 -> score;
}

void reorderAndSortList(VectorType<Move> *const restrict ml, const VectorType<Move> & putInFront, const search_data_t *const sd, const PlayerColor c)
{
        if (!ml -> empty()) {
		const Board *const b = sd -> s -> getBoard();

		const size_t pifs = putInFront.size(), mls = ml -> size();

		size_t putIndex = 0;
		for(size_t j=0; j<pifs; j++) {
			for(size_t i=putIndex; i<mls; i++) {
				Move *const m = ml -> atPointer(i);

				if (moveCompareTo(*m, putInFront.at(j))) {
					m -> score = MS_INF - 1 - j;
					ml -> swap(i, putIndex++);
					break;
				}
			}
		}

		for(size_t i=putIndex; i<mls; i++) {
			Move *const m = ml -> atPointer(i);

			m -> score = calcSortValue(m, b, sd, sd -> s -> gamePhase());
		}

//		printf("%zu %zu\n", putIndex, mls - putIndex);
		ml -> sort(moveSortVictimType, putIndex, mls);

#ifdef _DEBUG
		for(size_t i=1; i<ml -> size(); i++) {
			my_assert(ml -> atPointer(i) -> score <= ml -> atPointer(i - 1) -> score, "reorderAndSortList");
		}
#endif
	}
}
