// (C) Folkert van Heusden. Not to be distributed nor open-source.
// $Revision: 4080 $
#ifndef __PTT_H__
#define __PTT_H__

#include <atomic>
#include <random>
#include <stdint.h>

#include "Board.h"
#include "Utils.h"
#include "Gen.h"

#define ENDIANNESS 0x76543210

extern char REV_PTT[];

class IO;

typedef struct __PRAGMA_PACKED__
{
	uint64_t hash;

	union {
		struct {
			int16_t value, age;
		} _data;

		uint32_t data;
	} data_;
} pttEntry;

#define N_TE_PER_HASH_GROUP 8

typedef struct __PRAGMA_PACKED__
{
	pttEntry entries[N_TE_PER_HASH_GROUP];
} pttHashGroup;

void dumpPttEntry(const pttEntry *const p);

class Ptt final
{
private:
	const std::string name, path;

	pttHashGroup *entries;
	unsigned long int nEntries;

	size_t nBytesTruncated;
	bool isMmap;

	uint8_t age;

public:
	Ptt(const std::string & name, const uint64_t nBytes, const int nThreads, const std::string & path);
	~Ptt();

	void incAge() { age++; }

	bool lookup(const uint64_t board_hash, pttEntry *const te);
	void store(const uint64_t board_hash, const int d, const int value);

	int hashFullPermill();
};

#endif
