// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 5116 $
#ifndef __SCENE_H__
#define __SCENE_H__

extern char REV_SCENE[];

// depth 6 from initial position gave 47
// moves in the list on average
// maybe there's a difference for white
// and black?
#define INITIAL_MOVE_LIST_LENGTH 64

// expected number of plies it'll do in a search FIXME general include fil
// sometimes also total number of moves
#define INITIAL_SCENE_LENGTH 32

#include "Board.h"
#include "ChessPiece.h"

extern uint64_t genml, valml;

typedef struct
{
	VectorType<Move> lists[2]; // 0 = white
} MoveLists;

typedef struct
{
	struct
	{
		PlayerColor lastOpponentColor;
		Move lastOpponentMove;
	} lastMove;

	uint32_t isKingPinnedBitmap;

	bool wQMoved, bQMoved, isEndGame;

	bool ccheckedField[2], cfieldUnderAttack[2];

	int rule50, nFullMoves; // for FEN

	int materialCounts[2][6];

	uint32_t tcs[64];
} SceneState;

class Scene final
{
private:
	std::vector<SceneState> ss;
	SceneState *css;

	std::vector<MoveLists *> moveLists;
	int mlStateDepth;
	//
	Board b;
	ChessPiece pieces[2][16];
	ChessPiece *kings[2];

	void ConvertFEN(const char * const restrict fen, const char * const restrict castlingAvailability, const char *const restrict halfMoveCounter, const char *const restrict fullMoveCounter);
	void genBoard();
	void setupPieces();
	void findKings();
	void pushState();
	void popState();
	void allocateMoveLists();
	void allocateMoveListPair();
	void initializeMembers();
	void countMaterial();

	bool compareStateTo(const SceneState *const ss);
	bool isInsufficientMaterialDraw(int a[], int b[]);

public: // FIXME protected:
	void updateBoard();

	int getNHalfMoves() const { return ss.back().rule50; }
	int getNFullMoves() const { return ss.back().nFullMoves; }

public:
	Scene();
	Scene(const std::string & fen);
	Scene(const ChessPiece * const restrict wIn, const ChessPiece * const restrict bIn);
	Scene(const Scene & sIn);
	~Scene();

	bool compareTo(Scene & other);

	void doMove(const Move & m, BoardState *const cbs);
	void undoMove();
	PlayerColor getLastOpponentMoveColor() const;
	const Move * getLastOpponentMove() const;

	void generateMovelists();
	void partialValidateMoves(const PlayerColor color);
	bool validateMoves(const PlayerColor color, const size_t moveIndex);
	VectorType<Move> *getMoveList(PlayerColor c) const;

	void setNullMove(Move *const nmrc, int *const hm);
	void unsetNullMove(const Move & nmm, const int hm);

	bool isKingUnderAttack(const PlayerColor kingColor) const;
	bool isFieldUnderAttack(const PlayerColor oc, const int x, const int y) const;
	bool isOpponentPawnAt(const int x, const int y, const PlayerColor hisColor) const;
	bool isCheckMate(const PlayerColor side) const;
	bool isStaleMate(const PlayerColor side) const;
	bool isInsufficientMaterialDraw();
	bool isEndGame() const;
	bool isMiddleGame() const;

	bool getIsProtected(const ChessPiece *const restrict cp) const;
	bool getIsUnderAttack(const ChessPiece *const restrict cp) const;

	bool getIsUnderAttackByPawn(const ChessPiece *const restrict cp) const;

	void getCastlingRights(bool *const restrict wl, bool *const restrict ws, bool *const restrict bl, bool *const restrict bs) const;

	void countMaterial(const ChessPiece * const restrict objs, int *const restrict counts);
	int countProtected(const PlayerColor c) const;
	int countUnderAttack(const PlayerColor c) const;

	ChessPiece *getKing(const PlayerColor c) const;
	const ChessPiece *getPieces(const PlayerColor c) const;
	Board * getBoard();
	const Board * getBoardConst() const { return &b; }

	void getKingPinnedCount(int *const whitePinnedN, int *const blackPinnedN) const;

	const int * getMaterialCounts(const PlayerColor c) const { return ss.back().materialCounts[c]; }

	bool getSEE(const int x, const int y, const ChessPiece *const attacker) const;
	int gamePhase();
};

#endif
