// (C) Folkert van Heusden. Not to be distributed nor open-source.
// $Revision: 4892 $
#ifndef __TPT_H__
#define __TPT_H__

#include <atomic>
#include <random>
#include <stdint.h>

#include "Board.h"
#include "Utils.h"
#include "Gen.h"
#include "config.h"

#define ENDIANNESS 0x76543210

extern uint64_t Random64[];

extern char REV_TPT[];

class IO;

typedef enum { NOTVALID = 0, EXACT = 1, LOWERBOUND = 2, UPPERBOUND = 3 } tptEntryFlag;

typedef struct __PRAGMA_PACKED__
{
	// FIXME do not store the bits that are used as an index
	// e.g. when there are 2^16 or more entries in the tpt, then
	// store only the upper 6 bytes
	uint64_t hash;
	union u {
		struct {
			int16_t value;
			uint8_t flags : 2; // is a tptEntryFlag!
			uint8_t age : 6;
			uint8_t depth : 7;
			bool nm : 1;
			Move::_u_::_m_ m;
		} _data;

		uint64_t data;
	} data_;
} tptEntry;

#define N_TE_PER_HASH_GROUP 8

typedef struct __PRAGMA_PACKED__
{
	tptEntry entries[N_TE_PER_HASH_GROUP];
} tptHashGroup;

typedef struct
{
	uint64_t newHash, cleanup;
	uint64_t pawnNewHash;
} zobrist_t;

#ifdef _DEBUG
typedef struct
{
	std::atomic_uint_fast64_t get_total, get_hit, get_avg_depth;
	std::atomic_uint_fast64_t store_total, store_new, store_update, store_full;
} tpt_usage_stats;
#endif

void dumpTptEntry(const tptEntry *const p);

typedef enum { TF_NONE, TF_LOAD, TF_LOAD_SAVE } tpt_file_mode_t;

class Tpt final
{
private:
	const std::string name;

	tptHashGroup *entries;
	unsigned long int nEntries;

	size_t nBytesTruncated;
	bool isMmap;

	IO *io;
#ifdef _DEBUG
	tpt_usage_stats **us;
#endif

	uint8_t age;
public:
	Tpt(const std::string & name, IO *const io, const uint64_t nBytes, const int nThreads);
	~Tpt();

	static uint64_t findCastlingRightsHash(const Board *const b);
	static inline uint64_t calcZobristDo(const int pos, const int type, const PlayerColor c);
	static inline uint64_t calcZobristDo(const ChessPiece *const obj, const int pos);

	void registerIO(IO *const io);
	void setSize(const uint64_t nBytes, const int nThreads);
	int hashFullPermill();

	void incAge() { age++; }

	uint64_t getWhiteSwitch() const { return Random64[780]; }

	static zobrist_t updateZobrist(const zobrist_t & in, const BoardState & bs, const Board *const b, const PlayerColor c, const Move epMove);
	static zobrist_t updateZobristNullMove(const zobrist_t & in, const Board *const b, const PlayerColor c);
	static zobrist_t calcZobrist(const Board * const b, const PlayerColor c, const Move epMove);

	bool lookup(const uint64_t board_hash, tptEntry *const te);
	void store(const uint64_t board_hash, const tptEntryFlag f, const int d, const int value, const Move::_u_::_m_& m, const bool usenm);

	void verify();

	void reset();
};

#endif
