#include <stdio.h>
#include <string>
#include <vector>

#include "error.h"
#include "Tune.h"
#include "Utils.h"
#include "config.h"
#include "IO.h"

Tune::Tune(const std::string & filename)
{
	defaultSettings();

	log(ll_ok, "Tuning: loading %s", filename.c_str());

	// load file
	FILE *fh = fopen(filename.c_str(), "r");
	if (!fh)
		error_exit(true, "Cannot open %s", filename.c_str());

	int ln = 0;
	while(!feof(fh)) {
		char line[4096];
		if (!fgets(line, sizeof line, fh))
			break;

		ln++;
		std::vector<std::string> *parts = split(line, " ");

		if (parts -> size() == 2) {
			std::string key = parts -> at(0);
			int val = atoi(parts -> at(1).c_str());

			if (!setParam(key, val))
				error_exit(false, "Line %d: unknown key word \"%s\"", ln, parts -> at(0).c_str());
		}
		else {
			error_exit(false, "Line %d (\"%s\") invalid", ln, line);
		}

		delete parts;
	}
}

Tune::Tune()
{
	defaultSettings();
}

Tune::~Tune()
{
}

void Tune::defaultSettings()
{
	aspiration_window_size = ASPIRATION_WINDOW_SIZE;
	divSumPositionValue = 4;
	mulObjValue = 1;
	mulROF = 15;
	mulDoublePawn = 1;
	contemptEndgame = 0;
	contemptMiddlegame = 25;
	lmr1sub = 2, lmr1mul = 1, lmr1div = 1;
	lmr2sub = 1, lmr2mul = 2, lmr2div = 3;
	lmrStart = 4, lmrNext = 2;
	lmrMinDepth = 3;
	minThinkTime = 1;
	checkExtension = 1;
	mobilityMul = 10;
	moreBishops = 25, noPawns = -10, tooMuchPawns = -10;
	iidMinDepth = 5, iidReduce = 2;
	pawnMobilMul = 1, knightMobilMul = 1, bishopMobilMul = 1, rookMobilMul = 1, queenMobilMul = 1, kingMobilMul = 1;
	kingSafety = 3;
	centerControl = 10;
	pawnShield = 10;
	roer = 10;
	tempo = 10;
	mulIsolatedPawns = 10;
}

bool Tune::setParam(const std::string & key, const int val)
{
	if (key == "aspiration_window_size")
		aspiration_window_size = val;
	else if (key == "divSumPositionValue")
		divSumPositionValue = val;
	else if (key == "mulObjValue")
		mulObjValue = val;
	else if (key == "mulROF")
		mulROF = val;
	else if (key == "mulDoublePawn")
		mulDoublePawn = val;
	else if (key == "contemptEndgame")
		contemptEndgame = val;
	else if (key == "contemptMiddlegame")
		contemptMiddlegame = val;
	else if (key == "minThinkTime")
		minThinkTime = val;
	else if (key == "checkExtension")
		checkExtension = val;
	else if (key == "lmr1sub")
		lmr1sub = val;
	else if (key == "lmr1mul")
		lmr1mul = val;
	else if (key == "lmr1div")
		lmr1div = val;
	else if (key == "lmr2sub")
		lmr2sub = val;
	else if (key == "lmr2mul")
		lmr2mul = val;
	else if (key == "lmr2div")
		lmr2div = val;
	else if (key == "lmrStart")
		lmrStart = val;
	else if (key == "lmrNext")
		lmrNext = val;
	else if (key == "lmrMinDepth")
		lmrMinDepth = val;
	else if (key == "mobilityMul")
		mobilityMul = val;
	else if (key == "moreBishops")
		moreBishops = val;
	else if (key == "noPawns")
		noPawns = val;
	else if (key == "iidMinDepth")
		iidMinDepth = val;
	else if (key == "iidReduce")
		iidReduce = val;
	else if (key == "pawnMobilMul")
		pawnMobilMul = val;
	else if (key == "knightMobilMul")
		knightMobilMul = val;
	else if (key == "bishopMobilMul")
		bishopMobilMul = val;
	else if (key == "rookMobilMul")
		rookMobilMul = val;
	else if (key == "queenMobilMul")
		queenMobilMul = val;
	else if (key == "kingMobilMul")
		kingMobilMul = val;
	else if (key == "tooMuchPawns")
		tooMuchPawns = val;
	else if (key == "centerControl")
		centerControl = val;
	else if (key == "pawnShield")
		pawnShield = val;
	else if (key == "kingSafety")
		kingSafety = val;
	else if (key == "rookOnEndRanks")
		roer = val;
	else if (key == "tempo")
		tempo = val;
	else if (key == "mulIsolatedPawns")
		mulIsolatedPawns = val;
	else
		return false;

	log(ll_ok, "Tuning: set parameter %s to %d", key.c_str(), val);

	return true;
}

Tune *tune = NULL;

void initTune(const std::string & filename)
{
	if (filename.empty())
		tune = new Tune();
	else
		tune = new Tune(filename);
}

void freeTune()
{
	delete tune;
	tune = NULL;
}
