// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4487 $
#ifndef __UTILS_H__
#define __UTILS_H__

#include <string>
#include <vector>

#include "config.h"
#include "PlayerColor.h"
#include "ChessPiece.h"
#include "Scene.h"
#include "Tweaks.h"

#define MAX_N_THREADS_PER_GROUP 8

#define mymin(x, y) ((x) < (y) ? (x) : (y))
#define mymax(x, y) ((x) > (y) ? (x) : (y))

#define opponentColor(c) ((PlayerColor)!(int)c)

ChessPiece *createNewChessPiece(const ChessPieceType cpt, const PlayerColor, const int x, const int y, const bool isFirstMove);
std::string genFen(const Scene * const s, const bool shortOutput, const PlayerColor c);
bool SANToMove(const std::string & sanMove, const Scene & s, const PlayerColor myColor, const VectorType<Move> *const validMoves, Move *const m);

uint64_t get_ts_ms();

std::vector<std::string> * split(std::string in, std::string splitter);
std::string trim(std::string in);
std::string replace(std::string in, char from, char to);
std::string myformat(const char *const fmt, ...);

ssize_t read_retry(int fd, void *data, size_t len);
uint64_t genRnd();

unsigned int upper_power_of_two(unsigned int v);
uint32_t adler32(const void *const buf, const size_t buflength);

void * getCPUAffinity();
void setCPUAffinity(const int i);
void resetCPUAffinity(void *p);

#if defined(USE_PTHREAD_RW_LOCK) || defined(__APPLE__) || defined(NO_SHARED_MUTEX)
#include <pthread.h>

#define rw_lock pthread_rwlock_t
#define r_lock(x) pthread_rwlock_rdlock(x)
#define w_lock(x) pthread_rwlock_wrlock(x)
#define r_ulock(x) pthread_rwlock_unlock(x)
#define w_ulock(x) pthread_rwlock_unlock(x)
#else
#include <shared_mutex>

#define rw_lock std::shared_timed_mutex
#define r_lock(x) (x) -> lock_shared()
#define w_lock(x) (x) -> lock()
#define r_ulock(x) (x) -> unlock_shared()
#define w_ulock(x) (x) -> unlock()
#endif

rw_lock *alloc_rw_lock();
void free_rw_lock(rw_lock *const l);

std::string mytolower(const std::string & in);
bool yesOrNo(const std::string & in);

uint64_t int_from_mem(const uint8_t *p, int l);

std::string * getLineFromSTDIN();

int countBits1(const uint32_t in) ;

int WRITE(const int fd, const uint8_t *p, size_t l);
int start_listen(int portnr);
std::string get_endpoint_name(int fd);

#endif
