// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 3159 $
#ifndef __VectorType_H__
#define __VectorType_H__

#include "Tweaks.h"

#include <stdlib.h>
#include <string.h>
#include <utility>

template <class T>
class VectorType
{
private:
	T *items;
	size_t nIn, listSize;

public:
	inline VectorType() : items(NULL), nIn(0), listSize(0)
	{
	}

	VectorType(const VectorType & source) : items(NULL), nIn(0), listSize(0)
	{
		assign(source, false);
	}

	~VectorType()
	{
		free(items);
		items = NULL;
	}

	void assign(const VectorType & source, const bool reverse)
	{
		size_t newNIn = source.size();
		unsigned long int newNInBytes = newNIn * sizeof(T);

		unsigned long int currentBytes = nIn * sizeof(T);

		if (newNInBytes > currentBytes) {
			free(items);
			items = (T *)malloc(newNInBytes);

			listSize = nIn = newNIn;
		}
		else {
			nIn = newNIn;
		}

		const T *const p = source.raw();

		if (reverse) {
			const size_t sub = nIn - 1;

			for(size_t i=0; i<nIn; i++)
				items[sub - i] = p[i];
		}
		else if (newNInBytes) {
			memcpy(items, p, newNInBytes);
		}
	}

	inline const T *raw() const
	{
		return items;
	}

	inline void clear()
	{
		nIn = 0;
	}

	inline void insert(const T & item)
	{
		if (listSize == nIn)
		{
			listSize = listSize ? listSize * 2 : 4;

			items = (T *)realloc(items, sizeof(T) * listSize);
		}

		memmove(&items[1], &items[0], sizeof(T) * nIn);
		items[0] = item;
		nIn++;
	}

	inline void push_back(const T & item)
	{
		if (listSize == nIn)
		{
			listSize = listSize ? listSize * 2 : 4;

			items = (T *)realloc(items, sizeof(T) * listSize);
		}

		items[nIn++] = item;
	}

	inline void setSize(const size_t nElements)
	{
		listSize = nElements;

		items = (T *)realloc(items, sizeof(T) * listSize);
	}

	inline const T at(const size_t idx) const
	{
		return items[idx];
	}

	inline T *atPointer(const size_t idx)
	{
		return &items[idx];
	}

	inline void set(const size_t idx, const T & t)
	{
		items[idx] = t;
	}

	inline const T back() const
	{
		return items[nIn - 1];
	}

	inline void erase(const size_t idx)
	{
		if (idx < nIn - 1) {
			items[idx] = items[nIn - 1];
			//memmove(&items[idx], &items[idx + 1], (nIn - 1 - idx) * sizeof(T));
		}

		nIn--;
	}

	inline size_t size() const
	{
		return nIn;
	}

	inline bool empty() const
	{
		return nIn == 0;
	}

	inline void swap(const size_t a, const size_t b)
	{
		std::swap(items[a], items[b]);
	}

	void sort(int (*compare_func)(const void *restrict m1, const void *restrict m2), const size_t startOffset, const size_t sl)
	{
		if (sl - startOffset > 16)
			qsort(&items[startOffset], sl - startOffset, sizeof(T), compare_func);
		else {
			for(size_t i = startOffset; i < sl; i++) {
				for(int j = i; j > startOffset; j--) {
					if (compare_func(&items[j], &items[j - 1]) >= 0)
						break;

					swap(j, j - 1);
				}
			}
		}
	}
};

#endif
