#include <fstream> 
#include <iostream>
#include <map>
#include <math.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <unistd.h>
#include <vector>

pid_t exec_with_pipe(const char *command, int *const wr, int *const rd)
{
	int pipe_to_proc[2], pipe_from_proc[2];

	pipe(pipe_to_proc);
	pipe(pipe_from_proc);

	pid_t pid = fork();
	if (pid == 0) {
		close(0);

		dup(pipe_to_proc[0]);
		close(pipe_to_proc[1]);
		close(1);
		close(2);
		dup(pipe_from_proc[1]);
		dup(pipe_from_proc[1]);
		close(pipe_from_proc[0]);

		execlp(command, command, (void *)NULL);
	}

	close(pipe_to_proc[0]);
	close(pipe_from_proc[1]);

	*wr = pipe_to_proc[1];
	*rd = pipe_from_proc[0];

	return pid;
}

bool get_eval(const std::string & fen, const bool white, const int fd_w, const int fd_r, double *const score)
{
	char buffer[4096];

	int len = snprintf(buffer, sizeof buffer, "position fen %s\neval\n", fen.c_str());
	write(fd_w, buffer, len);

	for(;;) {
		int n = read(fd_r, buffer, sizeof buffer - 1);
		if (n <= 0) {
			printf("%s\n", strerror(errno));
			return false;
		}

		buffer[n] = 0x00;

		char *p = strstr(buffer, "Total Evaluation:");

		if (p) {
			*score = atof(&p[18]);

			if (!white)
				*score = -*score;

			return true;
		}
	}
}

std::vector<std::string> * split(std::string in, std::string splitter)
{
	std::vector<std::string> *out = new std::vector<std::string>;
	size_t splitter_size = splitter.size();

	for(;;)
	{
		size_t pos = in.find(splitter);
		if (pos == std::string::npos)
			break;

		std::string before = in.substr(0, pos);
		out -> push_back(before);

		size_t bytes_left = in.size() - (pos + splitter_size);
		if (bytes_left == 0)
		{
			out -> push_back("");
			return out;
		}

		in = in.substr(pos + splitter_size);
	}

	if (in.size() > 0)
		out -> push_back(in);

	return out;
}

double sum(std::vector<double> a)
{
	double s = 0;

	for (int i = 0; i < a.size(); i++)
		s += a[i];

	return s;
}

double mean(std::vector<double> a)
{
	return sum(a) / a.size();
}

double sqsum(std::vector<double> a)
{
	double s = 0;

	for (int i = 0; i < a.size(); i++)
		s += pow(a[i], 2);

	return s;
}

double stdev(std::vector<double> nums)
{
	double N = nums.size();
	return pow(sqsum(nums) / N - pow(sum(nums) / N, 2), 0.5);
}

std::vector<double> operator-(std::vector<double> a, double b)
{
	std::vector<double> retvect;

	for (int i = 0; i < a.size(); i++)
		retvect.push_back(a[i] - b);

	return retvect;
}

std::vector<double> operator*(std::vector<double> a, std::vector<double> b)
{
	std::vector<double> retvect;

	for (int i = 0; i < a.size() ; i++)
		retvect.push_back(a[i] * b[i]);

	return retvect;
}

double pearsoncoeff(std::vector<double> X, std::vector<double> Y)
{
	return sum((X - mean(X))*(Y - mean(Y))) / (X.size()*stdev(X)* stdev(Y));
}

int main(int argc, char *argv[])
{
	FILE *fh = fopen(argv[1], "r");
	if (!fh) {
		fprintf(stderr, "Cannot open %s\n", argv[1]);
		return 1;
	}

	signal(SIGPIPE, SIG_IGN);
	signal(SIGCHLD, SIG_IGN);

	std::vector<double> embla, opponent;

	int fdw_sf, fdr_sf;
	pid_t pid_sf = exec_with_pipe("/usr/games/stockfish", &fdw_sf, &fdr_sf);
	int fdw_e, fdr_e;
	pid_t pid_e = exec_with_pipe("./Embla-test.sh", &fdw_e, &fdr_e);

	char buffer[4096];
	while(!feof(fh)) {
		fgets(buffer, sizeof buffer, fh);

		std::vector<std::string> *parts = split(buffer, " ");

		if (parts -> size() < 4) { // 6
			delete parts;
			continue;
		}

		std::string fen = parts -> at(0) + " " + parts -> at(1) + " " + parts -> at(2) + " " + parts -> at(3); // + " " + parts -> at(4) + " " + parts -> at(5);

		bool white = parts -> at(1) == "w" || parts -> at(1) == "W";

		double score_sf = 0;
		if (!get_eval(fen, white, fdw_sf, fdr_sf, &score_sf)) {
			printf("eval for stockfish failed\n");
			return 1;
		}

		double score_e = 0;
		if (!get_eval(fen, white, fdw_e, fdr_e, &score_e)) {
			printf("eval for embla failed\n");
			return 1;
		}

		embla.push_back(score_e);
		opponent.push_back(score_sf);

		delete parts;
	}

	printf("%f\n", pearsoncoeff(embla, opponent));

	kill(pid_sf, SIGKILL);
	kill(pid_e, SIGKILL);

	close(fdw_sf);
	close(fdr_sf);
	close(fdw_e);
	close(fdr_e);

	return 0;
}

