// (C) Folkert van Heusden. AGPL v3.0.
// $Revision: 4872 $
#include "Tweaks.h"
#include "config.h"
#ifdef WITH_MYSQL_BOOKS
#include <stdlib.h>
#include <string>
#include <vector>

#include "db.h"
#include "error.h"
#include "IO.h"

void init_db()
{
	if (mysql_library_init(0, NULL, NULL))
		error_exit(false, "Problem initializing MySQL library");
}

MYSQL * connect_to_database(const char *const server, const char *const user, const char *const password, const char *const database)
{
	MYSQL *handle = mysql_init(NULL);
	if (!handle)
		return NULL;

	if (!server)
		error_exit(false, "No MySQL server selected");

	if (!user || !password)
		error_exit(false, "No MySQL username/password set");

	if (!database)
		error_exit(false, "No MySQL databse selected");

	if (mysql_real_connect(handle, server, user, password, database, 0, NULL, 0) == 0)
		error_exit(false, "Failed to connect to database, reason: %s", mysql_error(handle));

	my_bool reconnect = true;
	mysql_options(handle, MYSQL_OPT_RECONNECT, &reconnect);

	return handle;
}

bool exec_query(MYSQL *const handle, const std::string & query)
{
	if (mysql_query(handle, query.c_str()))
	{
		log(ll_err, "Query '%s' failed, reason: %s", query.c_str(), mysql_error(handle));
		return false;
	}

	return true;
}

bool do_query(MYSQL *const handle, const std::string & query, std::vector<std::vector<std::string> > *const out)
{
	if (!exec_query(handle, query))
		return false;

	int field_count = mysql_field_count(handle);

	MYSQL_RES *res = mysql_use_result(handle);

	MYSQL_ROW row = NULL;
	while((row = mysql_fetch_row(res)) != NULL)
	{
		std::vector<std::string> out_row;

		for(int index=0; index<field_count; index++)
			out_row.push_back(row[index]);

		out -> push_back(out_row);
	}

	mysql_free_result(res);

	return true;
}

bool do_query(MYSQL *const handle, const std::string & query)
{
	return exec_query(handle, query);

}

int n_affected_rows(MYSQL *const handle)
{
	return mysql_affected_rows(handle);
}
#endif
