#include "Tweaks.h"
#include <ctype.h>
#include <fcntl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>

#include "Scene.h"

int count(const char board[8][8], const char which)
{
	int cnt = 0;

	for(int y=0; y<8; y++)
	{
		for(int x=0; x<8; x++)
			cnt += !!(board[x][y] == which);
	}

	return cnt;
}

void put(char board[8][8], const char which)
{
	for(;;)
	{
		int x = int(lrand48() % 8);
		int y = int(lrand48() % 8);

                if ((which == 'p' || which == 'P') && (y == 0 || y == 7))
                        continue;

		if (board[x][y] == 0x00)
		{
			board[x][y] = which;
			break;
		}
	}
}

int main(int argc, char *argv[])
{
	int fd = open("/dev/urandom", O_RDONLY);
	unsigned long int x;
	read(fd, &x, sizeof x);
	close(fd);
	srand48(x);

	const char whichWhite[] = "rnbqp"; // king is manually added
	const char whichBlack[] = "RNBQP"; // king is manually added

	for(;;)
	{
		char board[8][8];
		memset(board, 0x00, sizeof board);

		put(board, 'k');
		put(board, 'K');

		int n = int(lrand48() % (16 * 2 - 2));
		int nWhite = 0, nBlack = 0;

		for(int i=0; i<n; i++)
		{
			int cur = 0x00;

			for(;;)
			{
				if ((lrand48() & 1) == 1 && nWhite < 16)
					cur = whichWhite[lrand48() % (sizeof whichWhite - 1)];
				else
					cur = whichBlack[lrand48() % (sizeof whichBlack - 1)];

				int cur_pawn = count(board, cur == 'q' ? 'p' : 'P');

				if ((cur == 'p' || cur == 'P') && cur_pawn < 8)
					break;

				int cur_queen = count(board, islower(cur) ? 'q' : 'Q');
				int cur_rook = count(board, islower(cur) ? 'r' : 'R');
				int cur_bishop = count(board, islower(cur) ? 'b' : 'B');
				int cur_knight = count(board, islower(cur) ? 'n' : 'N');

				if (cur_pawn + cur_queen - 1 + cur_rook - 2 + cur_bishop - 2 + cur_knight - 2 < 8)
					break;
			}

			put(board, cur);
		}

		char out[128];
		int o = 0;

		for(int y=7; y>=0; y--)
		{
			int cnt = 0;

			for(int x=0; x<8; x++)
			{
				if (board[x][y] == 0x00)
					cnt++;
				else
				{
					if (cnt)
					{
						out[o++] = '0' + cnt;
						cnt = 0;
					}

					out[o++] = board[x][y];
				}
			}

			if (cnt)
				out[o++] = '0' + cnt;

			if (y != 0)
				out[o++] = '/';
		}

		out[o++] = ' ';
		PlayerColor c = WHITE;
		if (lrand48() & 1)
			out[o++] = 'w';
		else
		{
			c = BLACK;
			out[o++] = 'b';
		}
		out[o++] = ' ';

		o += sprintf(&out[o], "- - %d %d", int(lrand48() % 51), int(lrand48() % 51));
		out[o] = 0x00;

		Scene s(std::string(out, o));
		s.generateMovelists();

		if (!s.isKingUnderAttack(BLACK) && !s.isKingUnderAttack(WHITE))
		{
			printf("%s\n", out);
			break;
		}
	}

	return 0;
}
