// (C) 2017 by folkert van heusden, released under AGPL v3.0
#include <algorithm>
#include <atomic>
#include <assert.h>
#include <errno.h>
#include <fcntl.h>
#include <netdb.h>
#include <poll.h>
#include <pthread.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <string>
#include <string.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <sys/ioctl.h>
#include <sys/mman.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>

#include "error.h"
#include "http_server.h"
#include "IO.h"
#include "Utils.h"
#include "http_content.h"

void send_file(const int fd, const std::string & filename)
{
	FILE *fh = fopen(filename.c_str(), "rb");
	if (!fh)
		return;

	uint8_t buffer[4096];
	while(!feof(fh)) {
		int rc = fread(buffer, 1, sizeof buffer, fh);
		if (rc <= 0)
			break;

		WRITE(fd, buffer, rc);
	}

	fclose(fh);
}

void http_server::client_handler(int cfd)
{
	const std::string http_200_header_html =
			"HTTP/1.0 200\r\n"
			"Cache-Control: no-cache\r\n"
			"Pragma: no-cache\r\n"
			"Server: Embla " VERSION "\r\n"
			"Expires: thu, 01 dec 1994 16:00:00 gmt\r\n"
			"Content-Type: text/html\r\n"
			"Connection: close\r\n"
			"\r\n";

	const std::string http_200_header_css =
			"HTTP/1.0 200\r\n"
			"Cache-Control: no-cache\r\n"
			"Pragma: no-cache\r\n"
			"Server: Embla " VERSION "\r\n"
			"Expires: thu, 01 dec 1994 16:00:00 gmt\r\n"
			"Content-Type: text/css\r\n"
			"Connection: close\r\n"
			"\r\n";

	const std::string http_200_header_svg =
			"HTTP/1.0 200\r\n"
			"Cache-Control: no-cache\r\n"
			"Pragma: no-cache\r\n"
			"Server: Embla " VERSION "\r\n"
			"Expires: thu, 01 dec 1994 16:00:00 gmt\r\n"
			"Content-Type: image/svg+xml\r\n"
			"Connection: close\r\n"
			"\r\n";

	const std::string http_200_header_ico =
			"HTTP/1.0 200\r\n"
			"Cache-Control: no-cache\r\n"
			"Pragma: no-cache\r\n"
			"Server: Embla " VERSION "\r\n"
			"Expires: thu, 01 dec 1994 16:00:00 gmt\r\n"
			"Content-Type: image/vnd.microsoft.icon\r\n"
			"Connection: close\r\n"
			"\r\n";

	char request_headers[65536] = { 0 };
	unsigned int h_n = 0;

	struct pollfd fds[1] = { { cfd, POLLIN, 0 } };

	for(;;) {
		if (poll(fds, 1, 250) == 0)
			continue;

		if (h_n >= sizeof request_headers - 1) {
			log(ll_ok, "HTTP request header too large");
			break;
		}

		int rc = read(cfd, &request_headers[h_n], sizeof request_headers - h_n - 1);
		if (rc == -1)
		{
			if (errno == EINTR || errno == EAGAIN)
				continue;

			log(ll_ok, "error receiving request headers");
			return;
		}
		else if (rc == 0)
		{
			log(ll_ok, "error receiving request headers");
			return;
		}

		h_n += rc;

		request_headers[h_n] = 0x00;

		if (strstr(request_headers, "\r\n\r\n") || strstr(request_headers, "\n\n"))
			break;
	}

	std::string path;

	bool get = true;
	if (strncmp(request_headers, "GET ", 4) == 0) {
		get = true;
		path = &request_headers[4];
	}
	else if (strncmp(request_headers, "HEAD ", 5) == 0) {
		get = false;
		path = &request_headers[5];
	}
	else {
		log(ll_ok, "HTTP request: not HEAD nor GET");
		return;
	}

// FIXME HEAD	if (!get)
//		return;

	std::string host = "127.0.0.1";
	char *hosth = strstr(request_headers, "Host:");
	if (hosth) {
		char *lf = strchr(hosth, '\n');
		if (lf) *lf = 0x00;
		char *cr = strchr(hosth, '\r');
		if (cr) *cr = 0x00;
		char *colon = strchr(&hosth[6], ':');
		if (colon) *colon = 0x00;

		host = &hosth[6];
	}

	std::string::size_type pos = path.find(" ");
	if (pos != std::string::npos)
		path = path.substr(0, pos);

	log(ll_ok, "HTTP request from %s: %s", get_endpoint_name(cfd).c_str(), path.c_str());

	if (path == "/" || path == "/index.html") {
		WRITE(cfd, (const uint8_t *)http_200_header_html.c_str(), http_200_header_html.size());

		std::string::size_type posr = http_index_html.find("REPLACEME");
		if (posr == std::string::npos) {
			log(ll_err, "Invalid index.html data");
			return;
		}

		std::string page = http_index_html.replace(posr, 9, myformat("%s:%d", host.c_str(), ws_port));

		WRITE(cfd, (const uint8_t *)page.c_str(), page.size());
		//send_file(cfd, "index.html");
	}
	else if (path == "/stylesheet.css") {
		WRITE(cfd, (const uint8_t *)http_200_header_css.c_str(), http_200_header_css.size());

		WRITE(cfd, (const uint8_t *)http_stylesheet_css.c_str(), http_stylesheet_css.size());
		//send_file(cfd, "stylesheet.css");
	}
	else if (path == "/favicon.ico") {
		WRITE(cfd, (const uint8_t *)http_200_header_ico.c_str(), http_200_header_ico.size());

		WRITE(cfd, favicon_ico, favicon_ico_len);
		//send_file(cfd, "favicon.ico");
	}
	else if (path.size() > 12 && path.substr(0, 12) == "/chessppics/") {
		WRITE(cfd, (const uint8_t *)http_200_header_svg.c_str(), http_200_header_svg.size());

		std::string filename = "/usr/share/games/xboard/themes/default/" + path.substr(12);

		send_file(cfd, filename);
	}
	else {
		printf("file not found: %s\n", path.c_str());
	}
}

http_server::http_server(const int http_port, const int ws_port_in) : ws_port(ws_port_in)
{
	fd = start_listen(http_port);

	terminate = false;

	th = new std::thread(std::ref(*this));
}

http_server::~http_server()
{
	terminate = true;

	if (th) {
		th -> join();
		delete th;
	}

	close(fd);
}

void http_server::operator()()
{
	struct pollfd fds[] = { { fd, POLLIN, 0 } };

	std::vector<pthread_t> handles;

	for(;!terminate;) {
		int rc = poll(fds, 1, 250);
		if (rc == 0)
			continue;

		if (rc == -1) {
			log(ll_err, "poll failed: %s", strerror(errno));
			break;
		}

		int cfd = accept(fd, NULL, 0);
		if (cfd == -1) {
			log(ll_err, "accept failed: %s", strerror(errno));
			continue;
		}

		pid_t pid = fork();
		if (pid == 0) {
			close(fd);
			client_handler(cfd);
			close(cfd);
			exit(0);
		}
		else if (pid == -1) {
			log(ll_err, "fork failed: %s", strerror(errno));
		}

		close(cfd);
	}

	log(ll_ok, "HTTP server thread terminating");
}
