#include <pthread.h>
#include <list>

template <typename T> class work_queue
{ 
	std::list<T> q;
	pthread_mutex_t lock;
	pthread_cond_t cv;

public:
	work_queue() {
		lock = PTHREAD_MUTEX_INITIALIZER;
		cv = PTHREAD_COND_INITIALIZER;
	}

	~work_queue() {
	}

	void put(T item) {
		pthread_mutex_lock(&lock);
		q.push_back(item);
		pthread_mutex_unlock(&lock);

		pthread_cond_signal(&cv);
	}

	T get() {
		pthread_mutex_lock(&lock);

		while (q.empty())
			pthread_cond_wait(&cv, &lock);

		T item = q.front();
		q.pop_front();

		pthread_mutex_unlock(&lock);

		return item;
	}
};
