#ifndef __WS_H__
#define __WS_H__

#include "config.h"
#ifdef WSJS
#include "IO.h"
#include "work_queue.h"

#include <atomic>
#include <set>
#include <thread>
#include <websocketpp/config/asio_no_tls.hpp>
#include <websocketpp/server.hpp>

#include <jansson.h>
void add_json_key_value(json_t *const target, const std::string & key, const std::string & value);
void add_json_key_value(json_t *const target, const std::string & key, const ULI value);

class wsserver {
public:
	typedef websocketpp::connection_hdl connection_hdl;

	wsserver(const int port);
	~wsserver();

	void operator()();
	void update(const std::string & json, const bool remember);
	void on_http(connection_hdl hdl);
	void on_open(connection_hdl hdl);
	void on_close(connection_hdl hdl);

	size_t getNWatchers();

	void setDefault(const std::string & defIn) { def = defIn; }

private:
	std::string def;

	work_queue<std::string> work;
	std::atomic_bool stop_flag;
	std::thread *worker;
	void workerMethod();

	typedef websocketpp::server<websocketpp::config::asio> server;

	typedef server::message_ptr message_ptr;
	void on_message(server* s, websocketpp::connection_hdl hdl, message_ptr msg);

	void set_timer();
	void on_timer(websocketpp::lib::error_code const & ec);
	server::timer_ptr m_timer;

	typedef std::set<connection_hdl,std::owner_less<connection_hdl> > con_list;
	server m_endpoint;

	std::mutex con_lock;
	con_list m_connections;

	const int port;
	std::thread *ws_thread_handle;

	std::string latest_json;
};
#endif

#endif
